experi <- function(form, ds, dsname, target, modeller, details="", 
                   n=100, control=NULL,
                   class="class",
                   log=paste0("experi.log", ".", dsname),
                   seed=19,
                   nobs.test=NULL  # use the last "nobs.test" rows of "ds" as test data
		   )
{
  nomem <- FALSE
  if (dsname == "weather")
      nomem <- TRUE
  if (modeller=="wsrf" && grepl("[[:alnum:]]*c[[:alnum:]]*", details))
      nomem <- TRUE
  
  if (n==0) return(NULL)
  suppressPackageStartupMessages(require(pROC))
  
  start.time <- proc.time()
  
  accs <- elapseds <- mems <- elapseds.pre <- NULL
  
  set.seed(seed, "default")
  seeds <- sample(1:1000000, n)
  
  for (i in seq_len(n))
  {
    set.seed(seeds[i], "default")

    if (is.null(nobs.test)) {
      train  <- sample(nrow(ds), 0.7*nrow(ds))
      test   <- setdiff(1:nrow(ds), train)
    } else {
      train  <- 1:(nrow(ds)-nobs.test)
      test   <- (nrow(ds)-nobs.test+1):nrow(ds)
    }
    actual <- ds[test, target]

    if (modeller == "bigrfc") {
      args <- append(list(x=ds[train, -which(names(ds)==target)], y=ds[train, target]), control)
    } else {
      args <- append(list(form, data=ds[train,]), control)
    }
    
    loop.time <- proc.time()
    
    if (nomem) {
      if (modeller == "bigrfc")
        capture.output(model  <- do.call(modeller, args), file="bigrf.build.log")
      else
        model  <- do.call(modeller, args)
      
      elapseds <- c((proc.time()-loop.time)['elapsed'], elapseds)
    } else if (modeller == "bigrfc") {
      if (dsname == "weather")
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.0000001)
      else if (dsname == "tis")
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.1)
      else
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.1)

      set.seed(seeds[i], "L'Ecuyer")
      capture.output(model  <- do.call(modeller, args), file="bigrf.build.log")
      Rprof(NULL)

      elapseds <- c((proc.time()-loop.time)['elapsed'], elapseds)
      mems <- c(max(summaryRprof(tf, memory="both")$by.total[, "mem.total"]), mems)

    } else {
      if (dsname == "weather")
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.0000001)
      else if ((dsname %in% c("fbis", "re1", "tis", "wap", "gisette")) && (grepl("nw", details, fixed=TRUE) || (modeller == "wsrf" && details == "b")))
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.01)
      else if (dsname == "mnist" && (details %in% c("nw_b", "nw_b_sqrt")))
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.01)
      else if (dsname == "newsgroups" && details == "nw_b")
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.01)
      else if (modeller == "wsrf")
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.1)
      else
        Rprof(tf <- "rprof.log", memory.profiling=TRUE, interval=0.9)
    
      model  <- do.call(modeller, args)
      Rprof(NULL)
      
      elapseds <- c((proc.time()-loop.time)['elapsed'], elapseds)
      mems <- c(max(summaryRprof(tf, memory="both")$by.total[, "mem.total"]), mems)
    }
    
    
    if (modeller %in% c("ctree", "cforest")) class <- "response"

    loop.time <- proc.time()
    
    if (modeller == "bigrfc") {
      capture.output(cl <- predict(model, ds[test, -which(names(ds)==target)], ds[test, target]), file="bigrf.predict.log")
      accs <- c(1 - cl@testerr, accs)
    } else {
      cl <- predict(model, newdata=ds[test,], type=class)
      accs <- c(sum(cl==actual, na.rm=TRUE)/length(actual), accs)
    }

    elapseds.pre <- c((proc.time()-loop.time)['elapsed'], elapseds.pre)
    if (! is.null(log))
    {
      require(lubridate)
      if (! file.exists(log))
        write.table(data.frame(ds=NA,
                               model=NA,
                               details=NA,
                               acc=NA,
                               elap=NA,
                               pred=NA,
                               mem=NA),
                    file=log, sep=",", row.names=FALSE)
      write.table(data.frame(ds=dsname,
                             model=modeller,
                             details=details,
                             acc=round(accs[1], 4),
                             elap=round(elapseds[1], 4),
                             pred=round(elapseds.pre[1], 4),
                             mem=ifelse(nomem, NA, mems[1])),
                  file=log, sep=",", append=TRUE, col.names=FALSE, row.names=FALSE)
    }

    rm("train", "test", "model", "cl")
    gc()
  }

}

experiSummary <- function(dsname, log=paste0("experi.log.", dsname))
{
  if (file.exists(log))
  {
    elog <- read.csv(log)

    s <- ddply(elog, .(model, details), summarise, 
               acc.mean=round(mean(acc), 4),
               acc.sd=round(sd(acc), 4),
               elap.mean=round(mean(elap), 4),
               elap.sd=round(sd(elap), 4),
               mem.mean=round(mean(mem), 4),
               mem.sd=round(sd(mem), 4),
               n=length(model))
    modeller <- paste0(s$model, "_", s$details)
    s$model <- s$details <- NULL
    s <- cbind(s, modeller)
    colnames(s) <- c("acc", "acc.sd", "elap", "elap.sd",
                     "mem", "mem.sd", "n", "modeller")
  }
  else
    s <- NULL

  return(s)
}
