######################################################################
# Code for generating Figure 5.
#
# Plots for building a forest of trees whilst varying the number of
# threads and nodes.
######################################################################

# Load utils
source("../myutil.R")
source("../load.R")

# r holds all the results.
r <- experiSummary("../../../experiment-results/")
r <- subset(r, r$ds != "weather")

pls <- list()
dfs <- NULL
label <- c("Number of threads", "Number of nodes")
prefix <- c(paste0("^", "wsrf", "_100t"), paste0("^", "wsrf", "_100c"))

extractdf <- function(prefix, label) {
  middle <- "[[:digit:]]+"
  postfix <- c("$")
  df <- subset(r, grepl(paste0(prefix, middle, postfix), modeller))
  vv <- as.integer(gsub(postfix, "", gsub(prefix, "", df$modeller)))
  df <- cbind(df, vv)
  colnames(df)[ncol(df)] <- "x"
  df$modeller <- label
  return(df)
}

dfs <- mapply(extractdf, prefix, label, SIMPLIFY=FALSE)
dfs <- rbind(dfs[[1]], dfs[[2]])
dfs$ds <- factor(dfs$ds, levels=ds.order, ordered=TRUE)

shapelist <- c(fbis=0, gisette=1, la1s=2, la2s=3, mnist=4,
newsgroups=5, re1=6, tis=7, wap=8)
linelist <- c(fbis=0, gisette=1, la1s=2, la2s=3, mnist=4,
newsgroups=5, re1=6, tis=7, wap=8)
colorlist <- c(fbis="red", gisette="blue", la1s="salmon",
la2s="orange", mnist="pink", newsgroups="green", re1="brown",
tis="purple", wap="yellow")
p <- ggplot(dfs, aes(x=x, y=elap))
p <- p + geom_line(aes(colour=ds, linetype=ds))
p <- p + geom_point(aes(colour=ds, shape=ds))
p <- p + scale_colour_manual("Dataset", values=colorlist)
p <- p + scale_linetype_manual("Dataset", values=linelist)
p <- p + scale_shape_manual("Dataset", values=shapelist)
p <- p + scale_y_continuous(labels = comma, trans="log10")
p <- p + scale_x_discrete(breaks=seq(2,19,by=3))
p <- p + labs(x="", y="Elapsed time (seconds)")
p <- p + facet_wrap(~ modeller, ncol=2, scales="free")
p <- p + theme(text=element_text(family="Times"),  # Using the default text font of JSS
               legend.text=element_text(family="Courier")
               )

grid.arrange(p, ncol=1)
