######################################################################
# Set up for all scripts of the manuscript.
######################################################################
# Load necessary packages
library("xtable")
library("plyr")
library("ggplot2")
library("gridExtra")
library("scales")


experiSummary <- function(log="../../../experiment-results/")
{
  logs <- unlist(strsplit(system(paste0("ls ", log), intern=TRUE),
                          "[ \n\t]+"))

  if (!is.null(logs))
  {
    elog <- NULL
    for (f in logs) elog <- rbind(elog, read.csv(paste0(log, f)))

    s <- ddply(elog, .(ds, model, details), summarise, 
               acc.mean=round(mean(acc), 4),
               elap.mean=round(mean(elap), 4),
               elap.tot=round(sum(elap), 4),
               mem.mean=round(mean(mem), 4),
               n=length(ds))
    modeller <- paste0(s$model, "_", s$details)
    s$model <- s$details <- NULL
    s <- cbind(s, modeller)
    colnames(s) <- c("ds",
                     "acc",
                     "elap",
                     "elap.tot",
                     "mem",
                     "n",
                     "modeller")
  } else
    s <- NULL

  return(s)
}


# Keep the same order for dataset presentation.
if (!exists("ds.order"))
  ds.order <- as.factor(c("wap",
                          "la1s",
                          "la2s",
                          "re1",
                          "fbis",
                          "gisette",
                          "newsgroups",
                          "tis",
                          "mnist"))

