######################################################################
# Code for Section 5 -- Package overview and usage of the manuscript.
######################################################################

# Load weather data from rattle
library("rattle")

# Set up the weather data
ds <- weather
target <- "RainTomorrow"
nobs <- nrow(ds)

# Check basic data information
dim(ds)
names(ds)

# Prepare
vars <- setdiff(names(ds), c("Date", "Location", "RISK_MM"))
dim(ds[vars])

# Deal with missing values
ds[vars] <- randomForest::na.roughfix(ds[vars])

table(ds[target])

(form <- formula(paste(target, "~ .")))

# Set seed for replication and prepare training and test data
set.seed(49)
train <- sample(nobs, 0.7 * nobs)
test  <- setdiff(1:nobs, train)
length(train)
length(test)

# Build wsrf model
library("wsrf")
system.time(model.wsrf <- wsrf(form, data = ds[train, vars]))

model <- model.wsrf

# Print wsrf model
print(model, tree = 1)
print(model, tree = 500)
print(model)

# Obtain strength and correlaton
strength(model)
correlation(model)

# Prediction
cl <- predict(model, newdata = ds[test, vars], type = "response")
actual <- ds[test, target]
(accuracy.wsrf <- sum(cl == actual) / length(actual))

# Build randomForest model
library("randomForest")
system.time(model.rf <- randomForest(form, data = ds[train, vars]))
model <- model.rf
model

cl <- predict(model, newdata = ds[test, vars], type = "response")
(accuracy.rf <- sum(cl == actual) / length(actual))

# Build model by party
library("party")
system.time(model.cf <- cforest(form, data = ds[train, vars]))
model <- model.cf
cl <- predict(model, newdata = ds[test, vars], type = "response")
(accuracy.cf <- sum(cl == actual) / length(actual))

# Identify available cores on the machine
detectCores()

# Do parallel computation

system.time(model.wsrf <- wsrf(form, data = ds[train, vars], parallel = 4))
print(model.wsrf)

# Distributed model building

# NB: The R code commented below need to be adapted according to the
# actual configuration before execution.

# servers <- c("node33", "node34", "node35", "node36")
# system.time(model.wsrf.1 <- wsrf(form, data = ds[train, vars], parallel = servers))
# print(model.wsrf.1)

# servers <- c(node33 = 2, node34 = 4, node35 = 8, node36 = 10)
# system.time(model.wsrf.2 <- wsrf(form, data = ds[train, vars], parallel = servers))
# print(model.wsrf.2)

# Model manipulation
model.subset  <- subset.wsrf(model.wsrf, 1:200)
model.combine <- combine.wsrf(model.wsrf.1, model.wsrf.2)


